;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC12F508. This file contains the basic code      *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    SS-AB-P12F508.asm                                 *
;    Date:          11.07.2014                                        *
;    File Version:  V2                                                *
;                                                                     *
;    Author:        Werner Hirsch                                     *
;    E-mail:        wernerhirschhh@aol.com                            *
;    Web:			www.wernerhirsch.de                                                                 * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=12F508            ; list directive to define processor
	#include <p12F508.inc>        ; processor specific variable definitions

	ERRORLEVEL      -302		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -305		;SUPPRESS BANK SELECTION MESSAGES
	ERRORLEVEL      -306		;SUPPRESS BANK SELECTION MESSAGES

	__CONFIG   _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file. 
; See respective data sheet for additional information on configuration word.
;
;===========================
;  DEFINITIONEN
;===========================
#define OUT_MUTE		GPIO, 0		;OUT_MUTE liegt an Pin GP0 (Pin 7)
#define IN_DETECT		GPIO, 1		;IN_DETECT liegt an Pin GP1 (Pin 6)
#define OUT_STBY		GPIO, 2		;OUT_STBY liegt an Pin GP2 (Pin 5)
#define IN_RESET		GPIO, 3		;IN_Reset liegt an Pin GP3 (Pin 4)
#define OUT_Vol_M		GPIO, 4		;OUT_Vol_M liegt an Pin GP4 (Pin 3)
#define IN_MUTE			GPIO, 5		;IN_MUTE liegt an Pin GP5 (Pin 2)



;===========================
;  VARIABLEN
;===========================
temp			EQU     0x07        ;example variable definition
loops			EQU     0x08        ;Warteschleife
loops2			EQU     0x09        ;Warteschleife2
det_loop		EQU     0x0A        ;Zhler fr Loops gesamt fr Detect(zur Reduzierung des Volumens)
det_num			EQU		0x0B		;Register fr Anzahl Detect (zur Reduzierung des Volumens)
det_num_max		EQU		0x0C		;Register fr maximale Anzahl Detect (zur Reduzierung des Volumens)
lock			EQU		0x0D		;Register, 1=locked, 0=unlocked
clip_count		EQU		0x0F		;Register fr Anzahl Clip (zur Abschaltung des Verstrkers)
clip_count_max	EQU		0x10		;Register fr maximale Anzahl Clip (zur Abschaltung des Verstrkers)
clip_1			EQU		0x11		;Speicherregister fr Clip Werte 1-8
clip_2			EQU		0x12		;Speicherregister fr Clip Werte 9-16
clip_bit		EQU		0x13		;Register fr Zwischenspeicherung des Bit Wertes 0/1
stat_amp		EQU		0x14		;Status Register fr Verstrker: 0 = off, 1 = on


;**********************************************************************
	ORG     0x1FF			; processor reset vector

; Internal RC calibration value is placed at location 0x1FF by Microchip
; as a movlw k, where the k is a literal value.

	ORG     0x000			; coding begins here
	movwf   OSCCAL			; update register with factory cal value 

	movlw	b'11010001'		; GP2 als I/O Pin gesetzt
	OPTION      			; update OPTION register

	movlw 	b'00101010'		;GP1,GP3,GP5 sind Eingnge, der Rest Ausgnge
	tris	GPIO			;

start	
	; Initialisierungen
	clrf GPIO				;Lsche alle Ausgangsports
	clrf det_loop			;Lsche Register fr Loop detect
	clrf det_num			;Lsche Register fr Anzahl detect
	clrf lock				;Lsche Register fr Lock
	clrf clip_count			;Lsche Register
	clrf clip_1				;Lsche Register
	clrf clip_2				;Lsche Register
	clrf clip_bit			;Lsche Register
	clrf stat_amp			;Lsche Register
	movlw d'75'				;lade wert 75 ins W-Register (75 von 250 = 30%)
	movwf det_num_max		;schiebe Wert ins F-Register	
	movlw d'8'				;lade wert 8 ins W-Register (8 von 16 = 50%)
	movwf clip_count_max	;schiebe Wert ins F-Register	

;Testbereich Anfang


;Testbereich Ende

	movlw d'250' 			;Wartezeit in ms (max 250) 250ms Gesamtwartezeit = 1s
	call WAIT				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call WAIT				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call WAIT				;Warteschleife
	movlw d'250' 			;Wartezeit in ms (max 250) 250ms
	call WAIT				;Warteschleife	



main						;Beginn der Hauptschleife
	btfsc IN_RESET			;Prfen, ob der Reset Taster gedrckt ist (=> Eingang low)
	goto $+4				;Taster ist nicht gedrckt, berspringe die nchsten 3 Befehle
	clrf lock				;Taster ist gedrckt, lsche lock bit
	clrf clip_1				;lsche Speicherregister
	clrf clip_2				;

	btfsc lock,0			;Prfen, ob das lock bit low ist
	goto main				;lock bit ist gesetzt, zurck zum Anfang der Hauptschleife

	btfss IN_MUTE			;Prfen, ob der Eingang fr die Amp Freigabe high ist (=> freigegeben)
	goto amp_off			;Eingang ist low (=> nicht freigegeben), weiter bei Verstrker OFF
	btfss IN_MUTE			;Nochmaliges Prfen, ob der Eingang fr die Amp Freigabe high ist (=> freigegeben)
	goto check_clip			;Eingang ist low (=> nicht freigegeben), weiter bei Programmteil fr Clip Auswertung
	goto amp_on				;Eingang ist high (=> freigegeben), weiter bei Verstrker ON

amp_off						;Unterprogramm fr Verstrker Abschaltung
	btfss stat_amp,0		;Prfen ob Statusbit gesetzt ist (0 = off, 1 = on)
	goto check_clip			;Statusbit ist bereits auf OFF, weiter bei Clip Auswertung
	clrf stat_amp			;Lsche Statusbit (und Register)
	bcf OUT_MUTE			;setze Ausgang MUTE auf low
	movlw d'50' 			;Wartezeit in ms (max 250) 50ms
	call WAIT				;Warteschleife
	bcf OUT_STBY			;setze Ausgang Stand By auf low
	goto check_clip

amp_on						;Unterprogramm fr Verstrker Freigabe
	btfsc stat_amp,0		;Prfen ob Statusbit nicht gesetzt ist (0 = off, 1 = on)
	goto check_clip			;Statusbit ist bereits auf on, weiter bei Clip Auswertung
	bsf stat_amp,0			;setze Statusbit fr ON
	bsf OUT_STBY			;setze Ausgang Stand By auf high
	movlw d'100' 			;Wartezeit in ms (max 250) 100ms
	call WAIT				;Warteschleife
	bsf OUT_MUTE			;setze ausgang Mute auf high


check_clip					;
	movlw d'240'
	subwf TMR0,w
	btfss STATUS,C
	goto main

	btfss IN_DETECT			;Prfen, ob das Signal fr IN_DETECT high ist
	incf det_num			;wenn nein, erhhe Zhler um 1

	decfsz det_loop			;reduziere Zhler um 1 und prfe ob 0
	goto check_clip_end		;Zhler ist noch nicht 0
	movlw d'250'			;Rcksetzen des Zhlers mit Wert 250
	movwf det_loop

	movfw det_num			;hole Zhler ins W-Register
	subwf det_num_max,W		;subtrahiere max Anzahl
	btfsc STATUS,C			;wenn ergebnis negativ, C=0 nchste Befehl, wenn ergebnis positiv, C=1 berspringe nchsten Befehl
	goto detect_no			;Zhler < max,
	goto detect_yes			;Zhler > max,
detect_no
	bcf OUT_Vol_M			;setze Ausgang fr Vol- auf low
	clrf det_num			;lschen des Zhlers
	clrf clip_bit			;lsche Bit
	goto check_average
detect_yes
	bsf OUT_Vol_M			;setze Ausgang fr Vol- auf high
	clrf det_num			;lschen des Zhlers
	bsf clip_bit,0			;setze Bit

check_average
	rlf	clip_1,f			;schiebe das Speicherregister 1 um 1 nach links
	rlf clip_2,f			;schiebe das Speicherregister 2 um 1 nach links
	bcf clip_1,0			;lsche das eingeschobene Bit
	btfsc clip_bit,0		;prfen ob clip_bit = 0
	bsf clip_1,0			;Bit ist 1, ersetze das eingeschobene Bit
	clrf clip_count			;lsche Register
	movfw clip_1			;Speicherregister 1+2 werden ausgezhlt
	call anzahl
	movfw clip_2			;
	call anzahl

	movfw clip_count		;hole Zhler ins W-Register
	subwf clip_count_max,w	;subtrahiere max Anzahl
	btfsc STATUS, C			;wenn ergebnis negativ, C=0 nchste Befehl, wenn ergebnis positiv, C=1 berspringe nchsten Befehl
	goto check_clip_end		;Ergebnis < max, weiter im Programm
	bcf OUT_MUTE			;Ergebnis > max, Abschalten des Amp, setze Ausgang fr Mute auf low
	movlw d'50' 			;Wartezeit in ms (max 250) 50ms
	call WAIT				;Warteschleife
	bcf OUT_STBY			;setze Ausgang Stand By auf low
	clrf stat_amp			;Lsche Statusbit (und Register)
	movlw d'1'				;setze lock bit in Register lock
	movwf lock				;	
	bcf OUT_Vol_M			;setze Ausgang fr Vol- auf low

check_clip_end
	movlw d'20' 			;Wartezeit, damit Timer 0 auf jeden Fall berluft (255 => 0)
	movwf loops				;Warteschleife
	decfsz loops
	goto $-1
		
	goto main				;zurck ins Hauptprogramm




;===========================
;  Unterprogramme
;===========================
 
WAIT						;Unterprogramm Warteschleife (max 250 ms)
	movwf   loops 
top
	movlw   d'250'			;die top-Schleife verzgert 1 ms = 250 x 4s
	movwf   loops2 
top2
	nop						;die top2-Schleife verzgert 4 Takte = 4 s
	decfsz  loops2, F  
	goto    top2 
	decfsz  loops, F 
	goto    top 
	retlw 0

anzahl
	movwf temp					;der zu analysierende wert wird ins temp Register kopiert
	movlw d'8'					;8 bits sollen analysiert werden
	movwf loops					;Zhler fr die auszuwertenden bits
rotieren
	rlf temp,f					;Das Zeichen wird nach links verschoben. Das Carryflag wird an den Ausgang weitergegeben
	btfsc STATUS,C				;Prfen, welchen Wert das Carryflag hat
	incf clip_count,f			;Carryflag = 1 -> Zhler um eins erhhen
	decfsz loops,f				;ein bit wurde bertragen -> Zhler um 1 verringern
	goto rotieren				;nchstes bit bertragen
	retlw 0		 
	

	END						; directive 'end of program'

